r'''
# netapp-fsxn-lun

> AWS CDK [L1 construct](https://docs.aws.amazon.com/cdk/latest/guide/constructs.html) and data structures for the [AWS CloudFormation Registry](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/registry.html) type `NetApp::FSxN::Lun` v1.1.0.

## Description

A LUN is a block storage object within a volume. It provides storage for applications that require block-level access and appears as a virtual disk to the connected host. It is formatted with a file system or used as raw storage, enabling precise control over storage resources and performance for specific workloads. Once activated, you need a preview key to consume this resource. Please reach out to Ng-fsx-cloudformation@netapp.com to get the key. To use this resource, you must first create the Link module.

## References

* [Source](https://github.com/NetApp/NetApp-CloudFormation-FSx-ONTAP-provider)

## Usage

In order to use this library, you will need to activate this AWS CloudFormation Registry type in your account. You can do this via the AWS Management Console or using the [AWS CLI](https://aws.amazon.com/cli/) using the following command:

```sh
aws cloudformation activate-type \
  --type-name NetApp::FSxN::Lun \
  --publisher-id a25d267c2b9b86b8d408fce3c7a4d94d34c90946 \
  --type RESOURCE \
  --execution-role-arn ROLE-ARN
```

Alternatively:

```sh
aws cloudformation activate-type \
  --public-type-arn arn:aws:cloudformation:us-east-1::type/resource/a25d267c2b9b86b8d408fce3c7a4d94d34c90946/NetApp-FSxN-Lun \
  --execution-role-arn ROLE-ARN
```

You can find more information about activating this type in the [AWS CloudFormation documentation](https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/registry-public.html).

## Feedback

This library is auto-generated and published to all supported programming languages by the [cdklabs/cdk-cloudformation](https://github.com/cdklabs/cdk-cloudformation) project based on the API schema published for `NetApp::FSxN::Lun`.

* Issues related to this generated library should be [reported here](https://github.com/cdklabs/cdk-cloudformation/issues/new?title=Issue+with+%40cdk-cloudformation%2Fnetapp-fsxn-lun+v1.1.0).
* Issues related to `NetApp::FSxN::Lun` should be reported to the [publisher](https://github.com/NetApp/NetApp-CloudFormation-FSx-ONTAP-provider).

## License

Distributed under the Apache-2.0 License.
'''
from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

import abc
import builtins
import datetime
import enum
import typing

import jsii
import publication
import typing_extensions

import typeguard
from importlib.metadata import version as _metadata_package_version
TYPEGUARD_MAJOR_VERSION = int(_metadata_package_version('typeguard').split('.')[0])

def check_type(argname: str, value: object, expected_type: typing.Any) -> typing.Any:
    if TYPEGUARD_MAJOR_VERSION <= 2:
        return typeguard.check_type(argname=argname, value=value, expected_type=expected_type) # type:ignore
    else:
        if isinstance(value, jsii._reference_map.InterfaceDynamicProxy): # pyright: ignore [reportAttributeAccessIssue]
           pass
        else:
            if TYPEGUARD_MAJOR_VERSION == 3:
                typeguard.config.collection_check_strategy = typeguard.CollectionCheckStrategy.ALL_ITEMS # type:ignore
                typeguard.check_type(value=value, expected_type=expected_type) # type:ignore
            else:
                typeguard.check_type(value=value, expected_type=expected_type, collection_check_strategy=typeguard.CollectionCheckStrategy.ALL_ITEMS) # type:ignore

from ._jsii import *

import aws_cdk as _aws_cdk_ceddda9d
import constructs as _constructs_77d1e7e8


class CfnLun(
    _aws_cdk_ceddda9d.CfnResource,
    metaclass=jsii.JSIIMeta,
    jsii_type="@cdk-cloudformation/netapp-fsxn-lun.CfnLun",
):
    '''A CloudFormation ``NetApp::FSxN::Lun``.

    :cloudformationResource: NetApp::FSxN::Lun
    :link: https://github.com/NetApp/NetApp-CloudFormation-FSx-ONTAP-provider
    '''

    def __init__(
        self,
        scope: _constructs_77d1e7e8.Construct,
        id: builtins.str,
        *,
        file_system_id: builtins.str,
        fsx_admin_password_source: typing.Union["PasswordSource", typing.Dict[builtins.str, typing.Any]],
        link_arn: builtins.str,
        name: builtins.str,
        os_type: "CfnLunPropsOsType",
        size: jsii.Number,
        svm: typing.Union["Svm", typing.Dict[builtins.str, typing.Any]],
        i_groups: typing.Optional[typing.Sequence[builtins.str]] = None,
        qos_policy_name: typing.Optional[builtins.str] = None,
    ) -> None:
        '''Create a new ``NetApp::FSxN::Lun``.

        :param scope: - scope in which this resource is defined.
        :param id: - scoped id of the resource.
        :param file_system_id: The file system ID of the Amazon FSx for NetApp ONTAP file system in which the resource is created.
        :param fsx_admin_password_source: The password source for the FSx admin user.
        :param link_arn: The ARN of the AWS Lambda function that will be invoked to manage the resource.
        :param name: The fully qualified path name of the LUN composed of a '/vol' prefix, the volume name, and base name of the LUN. Example: /vol/volume1/lun1
        :param os_type: The host operating system of the initiator group. All initiators in the group should be hosts of the same operating system.
        :param size: Size of the LUN. Required when creating a non-clone LUN and disallowed when creating a clone of an existing LUN. A clone's size is taken from the source LUN.
        :param svm: The SVM information associated with the LUN.
        :param i_groups: The initiator groups mapped to this LUN.
        :param qos_policy_name: The name of the QoS policy for the LUN.
        '''
        if __debug__:
            type_hints = typing.get_type_hints(_typecheckingstub__790cf609fcc39c2a96a3c5c4e15945905bfefbf5c380085f4f938bed5fdcbc3e)
            check_type(argname="argument scope", value=scope, expected_type=type_hints["scope"])
            check_type(argname="argument id", value=id, expected_type=type_hints["id"])
        props = CfnLunProps(
            file_system_id=file_system_id,
            fsx_admin_password_source=fsx_admin_password_source,
            link_arn=link_arn,
            name=name,
            os_type=os_type,
            size=size,
            svm=svm,
            i_groups=i_groups,
            qos_policy_name=qos_policy_name,
        )

        jsii.create(self.__class__, self, [scope, id, props])

    @jsii.python.classproperty
    @jsii.member(jsii_name="CFN_RESOURCE_TYPE_NAME")
    def CFN_RESOURCE_TYPE_NAME(cls) -> builtins.str:
        '''The CloudFormation resource type name for this resource class.'''
        return typing.cast(builtins.str, jsii.sget(cls, "CFN_RESOURCE_TYPE_NAME"))

    @builtins.property
    @jsii.member(jsii_name="attrUuid")
    def attr_uuid(self) -> builtins.str:
        '''Attribute ``NetApp::FSxN::Lun.UUID``.

        :link: https://github.com/NetApp/NetApp-CloudFormation-FSx-ONTAP-provider
        '''
        return typing.cast(builtins.str, jsii.get(self, "attrUuid"))

    @builtins.property
    @jsii.member(jsii_name="props")
    def props(self) -> "CfnLunProps":
        '''Resource props.'''
        return typing.cast("CfnLunProps", jsii.get(self, "props"))


@jsii.data_type(
    jsii_type="@cdk-cloudformation/netapp-fsxn-lun.CfnLunProps",
    jsii_struct_bases=[],
    name_mapping={
        "file_system_id": "fileSystemId",
        "fsx_admin_password_source": "fsxAdminPasswordSource",
        "link_arn": "linkArn",
        "name": "name",
        "os_type": "osType",
        "size": "size",
        "svm": "svm",
        "i_groups": "iGroups",
        "qos_policy_name": "qosPolicyName",
    },
)
class CfnLunProps:
    def __init__(
        self,
        *,
        file_system_id: builtins.str,
        fsx_admin_password_source: typing.Union["PasswordSource", typing.Dict[builtins.str, typing.Any]],
        link_arn: builtins.str,
        name: builtins.str,
        os_type: "CfnLunPropsOsType",
        size: jsii.Number,
        svm: typing.Union["Svm", typing.Dict[builtins.str, typing.Any]],
        i_groups: typing.Optional[typing.Sequence[builtins.str]] = None,
        qos_policy_name: typing.Optional[builtins.str] = None,
    ) -> None:
        '''A LUN is a block storage object within a volume.

        It provides storage for applications that require block-level access and appears as a virtual disk to the connected host. It is formatted with a file system or used as raw storage, enabling precise control over storage resources and performance for specific workloads. Once activated, you need a preview key to consume this resource. Please reach out to Ng-fsx-cloudformation@netapp.com to get the key. To use this resource, you must first create the Link module.

        :param file_system_id: The file system ID of the Amazon FSx for NetApp ONTAP file system in which the resource is created.
        :param fsx_admin_password_source: The password source for the FSx admin user.
        :param link_arn: The ARN of the AWS Lambda function that will be invoked to manage the resource.
        :param name: The fully qualified path name of the LUN composed of a '/vol' prefix, the volume name, and base name of the LUN. Example: /vol/volume1/lun1
        :param os_type: The host operating system of the initiator group. All initiators in the group should be hosts of the same operating system.
        :param size: Size of the LUN. Required when creating a non-clone LUN and disallowed when creating a clone of an existing LUN. A clone's size is taken from the source LUN.
        :param svm: The SVM information associated with the LUN.
        :param i_groups: The initiator groups mapped to this LUN.
        :param qos_policy_name: The name of the QoS policy for the LUN.

        :schema: CfnLunProps
        '''
        if isinstance(fsx_admin_password_source, dict):
            fsx_admin_password_source = PasswordSource(**fsx_admin_password_source)
        if isinstance(svm, dict):
            svm = Svm(**svm)
        if __debug__:
            type_hints = typing.get_type_hints(_typecheckingstub__ac69f1cc45b9a0c8d0c94c385659c73c7c3e3f92e5214dd45fea7a8174ac4b37)
            check_type(argname="argument file_system_id", value=file_system_id, expected_type=type_hints["file_system_id"])
            check_type(argname="argument fsx_admin_password_source", value=fsx_admin_password_source, expected_type=type_hints["fsx_admin_password_source"])
            check_type(argname="argument link_arn", value=link_arn, expected_type=type_hints["link_arn"])
            check_type(argname="argument name", value=name, expected_type=type_hints["name"])
            check_type(argname="argument os_type", value=os_type, expected_type=type_hints["os_type"])
            check_type(argname="argument size", value=size, expected_type=type_hints["size"])
            check_type(argname="argument svm", value=svm, expected_type=type_hints["svm"])
            check_type(argname="argument i_groups", value=i_groups, expected_type=type_hints["i_groups"])
            check_type(argname="argument qos_policy_name", value=qos_policy_name, expected_type=type_hints["qos_policy_name"])
        self._values: typing.Dict[builtins.str, typing.Any] = {
            "file_system_id": file_system_id,
            "fsx_admin_password_source": fsx_admin_password_source,
            "link_arn": link_arn,
            "name": name,
            "os_type": os_type,
            "size": size,
            "svm": svm,
        }
        if i_groups is not None:
            self._values["i_groups"] = i_groups
        if qos_policy_name is not None:
            self._values["qos_policy_name"] = qos_policy_name

    @builtins.property
    def file_system_id(self) -> builtins.str:
        '''The file system ID of the Amazon FSx for NetApp ONTAP file system in which the resource is created.

        :schema: CfnLunProps#FileSystemId
        '''
        result = self._values.get("file_system_id")
        assert result is not None, "Required property 'file_system_id' is missing"
        return typing.cast(builtins.str, result)

    @builtins.property
    def fsx_admin_password_source(self) -> "PasswordSource":
        '''The password source for the FSx admin user.

        :schema: CfnLunProps#FsxAdminPasswordSource
        '''
        result = self._values.get("fsx_admin_password_source")
        assert result is not None, "Required property 'fsx_admin_password_source' is missing"
        return typing.cast("PasswordSource", result)

    @builtins.property
    def link_arn(self) -> builtins.str:
        '''The ARN of the AWS Lambda function that will be invoked to manage the resource.

        :schema: CfnLunProps#LinkArn
        '''
        result = self._values.get("link_arn")
        assert result is not None, "Required property 'link_arn' is missing"
        return typing.cast(builtins.str, result)

    @builtins.property
    def name(self) -> builtins.str:
        '''The fully qualified path name of the LUN composed of a '/vol' prefix, the volume name, and base name of the LUN.

        Example: /vol/volume1/lun1

        :schema: CfnLunProps#Name
        '''
        result = self._values.get("name")
        assert result is not None, "Required property 'name' is missing"
        return typing.cast(builtins.str, result)

    @builtins.property
    def os_type(self) -> "CfnLunPropsOsType":
        '''The host operating system of the initiator group.

        All initiators in the group should be hosts of the same operating system.

        :schema: CfnLunProps#OsType
        '''
        result = self._values.get("os_type")
        assert result is not None, "Required property 'os_type' is missing"
        return typing.cast("CfnLunPropsOsType", result)

    @builtins.property
    def size(self) -> jsii.Number:
        '''Size of the LUN.

        Required when creating a non-clone LUN and disallowed when creating a clone of an existing LUN. A clone's size is taken from the source LUN.

        :schema: CfnLunProps#Size
        '''
        result = self._values.get("size")
        assert result is not None, "Required property 'size' is missing"
        return typing.cast(jsii.Number, result)

    @builtins.property
    def svm(self) -> "Svm":
        '''The SVM information associated with the LUN.

        :schema: CfnLunProps#SVM
        '''
        result = self._values.get("svm")
        assert result is not None, "Required property 'svm' is missing"
        return typing.cast("Svm", result)

    @builtins.property
    def i_groups(self) -> typing.Optional[typing.List[builtins.str]]:
        '''The initiator groups mapped to this LUN.

        :schema: CfnLunProps#IGroups
        '''
        result = self._values.get("i_groups")
        return typing.cast(typing.Optional[typing.List[builtins.str]], result)

    @builtins.property
    def qos_policy_name(self) -> typing.Optional[builtins.str]:
        '''The name of the QoS policy for the LUN.

        :schema: CfnLunProps#QosPolicyName
        '''
        result = self._values.get("qos_policy_name")
        return typing.cast(typing.Optional[builtins.str], result)

    def __eq__(self, rhs: typing.Any) -> builtins.bool:
        return isinstance(rhs, self.__class__) and rhs._values == self._values

    def __ne__(self, rhs: typing.Any) -> builtins.bool:
        return not (rhs == self)

    def __repr__(self) -> str:
        return "CfnLunProps(%s)" % ", ".join(
            k + "=" + repr(v) for k, v in self._values.items()
        )


@jsii.enum(jsii_type="@cdk-cloudformation/netapp-fsxn-lun.CfnLunPropsOsType")
class CfnLunPropsOsType(enum.Enum):
    '''The host operating system of the initiator group.

    All initiators in the group should be hosts of the same operating system.

    :schema: CfnLunPropsOsType
    '''

    AIX = "AIX"
    '''aix.'''
    HPUX = "HPUX"
    '''hpux.'''
    HYPER_UNDERSCORE_V = "HYPER_UNDERSCORE_V"
    '''hyper_v.'''
    LINUX = "LINUX"
    '''linux.'''
    NETWARE = "NETWARE"
    '''netware.'''
    OPENVMS = "OPENVMS"
    '''openvms.'''
    SOLARIS = "SOLARIS"
    '''solaris.'''
    SOLARIS_UNDERSCORE_EFI = "SOLARIS_UNDERSCORE_EFI"
    '''solaris_efi.'''
    VMWARE = "VMWARE"
    '''vmware.'''
    WINDOWS = "WINDOWS"
    '''windows.'''
    WINDOWS_UNDERSCORE_2008 = "WINDOWS_UNDERSCORE_2008"
    '''windows_2008.'''
    WINDOWS_UNDERSCORE_GPT = "WINDOWS_UNDERSCORE_GPT"
    '''windows_gpt.'''
    XEN = "XEN"
    '''xen.'''


@jsii.data_type(
    jsii_type="@cdk-cloudformation/netapp-fsxn-lun.PasswordSource",
    jsii_struct_bases=[],
    name_mapping={"secret": "secret"},
)
class PasswordSource:
    def __init__(
        self,
        *,
        secret: typing.Union["SecretSource", typing.Dict[builtins.str, typing.Any]],
    ) -> None:
        '''
        :param secret: A reference to the source of the password, typically an AWS Secrets Manager secret.

        :schema: PasswordSource
        '''
        if isinstance(secret, dict):
            secret = SecretSource(**secret)
        if __debug__:
            type_hints = typing.get_type_hints(_typecheckingstub__79f6bc3875ab0edeb62b92a4e90b1344d2b538a648833126779770466b39e04e)
            check_type(argname="argument secret", value=secret, expected_type=type_hints["secret"])
        self._values: typing.Dict[builtins.str, typing.Any] = {
            "secret": secret,
        }

    @builtins.property
    def secret(self) -> "SecretSource":
        '''A reference to the source of the password, typically an AWS Secrets Manager secret.

        :schema: PasswordSource#Secret
        '''
        result = self._values.get("secret")
        assert result is not None, "Required property 'secret' is missing"
        return typing.cast("SecretSource", result)

    def __eq__(self, rhs: typing.Any) -> builtins.bool:
        return isinstance(rhs, self.__class__) and rhs._values == self._values

    def __ne__(self, rhs: typing.Any) -> builtins.bool:
        return not (rhs == self)

    def __repr__(self) -> str:
        return "PasswordSource(%s)" % ", ".join(
            k + "=" + repr(v) for k, v in self._values.items()
        )


@jsii.data_type(
    jsii_type="@cdk-cloudformation/netapp-fsxn-lun.SecretSource",
    jsii_struct_bases=[],
    name_mapping={"secret_arn": "secretArn", "secret_key": "secretKey"},
)
class SecretSource:
    def __init__(self, *, secret_arn: builtins.str, secret_key: builtins.str) -> None:
        '''
        :param secret_arn: The ARN of the secret stored in AWS Secrets Manager.
        :param secret_key: Reference for the SecretKey. The actual password is stored in AWS Secret Manager.

        :schema: SecretSource
        '''
        if __debug__:
            type_hints = typing.get_type_hints(_typecheckingstub__1643a3d27c5d2df892db47b07541e26fc4d71cd0ed0757ff8cbd579540ddb512)
            check_type(argname="argument secret_arn", value=secret_arn, expected_type=type_hints["secret_arn"])
            check_type(argname="argument secret_key", value=secret_key, expected_type=type_hints["secret_key"])
        self._values: typing.Dict[builtins.str, typing.Any] = {
            "secret_arn": secret_arn,
            "secret_key": secret_key,
        }

    @builtins.property
    def secret_arn(self) -> builtins.str:
        '''The ARN of the secret stored in AWS Secrets Manager.

        :schema: SecretSource#SecretArn
        '''
        result = self._values.get("secret_arn")
        assert result is not None, "Required property 'secret_arn' is missing"
        return typing.cast(builtins.str, result)

    @builtins.property
    def secret_key(self) -> builtins.str:
        '''Reference for the SecretKey.

        The actual password is stored in AWS Secret Manager.

        :schema: SecretSource#SecretKey
        '''
        result = self._values.get("secret_key")
        assert result is not None, "Required property 'secret_key' is missing"
        return typing.cast(builtins.str, result)

    def __eq__(self, rhs: typing.Any) -> builtins.bool:
        return isinstance(rhs, self.__class__) and rhs._values == self._values

    def __ne__(self, rhs: typing.Any) -> builtins.bool:
        return not (rhs == self)

    def __repr__(self) -> str:
        return "SecretSource(%s)" % ", ".join(
            k + "=" + repr(v) for k, v in self._values.items()
        )


@jsii.data_type(
    jsii_type="@cdk-cloudformation/netapp-fsxn-lun.Svm",
    jsii_struct_bases=[],
    name_mapping={"name": "name", "uuid": "uuid"},
)
class Svm:
    def __init__(
        self,
        *,
        name: typing.Optional[builtins.str] = None,
        uuid: typing.Optional[builtins.str] = None,
    ) -> None:
        '''
        :param name: The name of the SVM.
        :param uuid: The UUID of the SVM.

        :schema: SVM
        '''
        if __debug__:
            type_hints = typing.get_type_hints(_typecheckingstub__bff3b67eed1a3f83f293867cc56efb9dd0095f2915af9df5cad3d7f23bf681ed)
            check_type(argname="argument name", value=name, expected_type=type_hints["name"])
            check_type(argname="argument uuid", value=uuid, expected_type=type_hints["uuid"])
        self._values: typing.Dict[builtins.str, typing.Any] = {}
        if name is not None:
            self._values["name"] = name
        if uuid is not None:
            self._values["uuid"] = uuid

    @builtins.property
    def name(self) -> typing.Optional[builtins.str]:
        '''The name of the SVM.

        :schema: SVM#Name
        '''
        result = self._values.get("name")
        return typing.cast(typing.Optional[builtins.str], result)

    @builtins.property
    def uuid(self) -> typing.Optional[builtins.str]:
        '''The UUID of the SVM.

        :schema: SVM#UUID
        '''
        result = self._values.get("uuid")
        return typing.cast(typing.Optional[builtins.str], result)

    def __eq__(self, rhs: typing.Any) -> builtins.bool:
        return isinstance(rhs, self.__class__) and rhs._values == self._values

    def __ne__(self, rhs: typing.Any) -> builtins.bool:
        return not (rhs == self)

    def __repr__(self) -> str:
        return "Svm(%s)" % ", ".join(
            k + "=" + repr(v) for k, v in self._values.items()
        )


__all__ = [
    "CfnLun",
    "CfnLunProps",
    "CfnLunPropsOsType",
    "PasswordSource",
    "SecretSource",
    "Svm",
]

publication.publish()

def _typecheckingstub__790cf609fcc39c2a96a3c5c4e15945905bfefbf5c380085f4f938bed5fdcbc3e(
    scope: _constructs_77d1e7e8.Construct,
    id: builtins.str,
    *,
    file_system_id: builtins.str,
    fsx_admin_password_source: typing.Union[PasswordSource, typing.Dict[builtins.str, typing.Any]],
    link_arn: builtins.str,
    name: builtins.str,
    os_type: CfnLunPropsOsType,
    size: jsii.Number,
    svm: typing.Union[Svm, typing.Dict[builtins.str, typing.Any]],
    i_groups: typing.Optional[typing.Sequence[builtins.str]] = None,
    qos_policy_name: typing.Optional[builtins.str] = None,
) -> None:
    """Type checking stubs"""
    pass

def _typecheckingstub__ac69f1cc45b9a0c8d0c94c385659c73c7c3e3f92e5214dd45fea7a8174ac4b37(
    *,
    file_system_id: builtins.str,
    fsx_admin_password_source: typing.Union[PasswordSource, typing.Dict[builtins.str, typing.Any]],
    link_arn: builtins.str,
    name: builtins.str,
    os_type: CfnLunPropsOsType,
    size: jsii.Number,
    svm: typing.Union[Svm, typing.Dict[builtins.str, typing.Any]],
    i_groups: typing.Optional[typing.Sequence[builtins.str]] = None,
    qos_policy_name: typing.Optional[builtins.str] = None,
) -> None:
    """Type checking stubs"""
    pass

def _typecheckingstub__79f6bc3875ab0edeb62b92a4e90b1344d2b538a648833126779770466b39e04e(
    *,
    secret: typing.Union[SecretSource, typing.Dict[builtins.str, typing.Any]],
) -> None:
    """Type checking stubs"""
    pass

def _typecheckingstub__1643a3d27c5d2df892db47b07541e26fc4d71cd0ed0757ff8cbd579540ddb512(
    *,
    secret_arn: builtins.str,
    secret_key: builtins.str,
) -> None:
    """Type checking stubs"""
    pass

def _typecheckingstub__bff3b67eed1a3f83f293867cc56efb9dd0095f2915af9df5cad3d7f23bf681ed(
    *,
    name: typing.Optional[builtins.str] = None,
    uuid: typing.Optional[builtins.str] = None,
) -> None:
    """Type checking stubs"""
    pass
